/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RECT_H
#define RECT_H

#include <nds/arm9/video.h>

class Rect {
public:
	int x, y, w, h;

	Rect(void) : x(0), y(0), w(0), h(0) { }
	Rect(int xx, int yy, int width, int height) : x(xx), y(yy), w(width), h(height) { }

	inline void Set(int xx, int yy, int width, int height) { x = xx; y = yy; w = width; h = height; }

	inline void Clear(void) { x = y = w = h = 0; }
	inline bool IsEmpty(void) { return (w == 0 && h == 0 && x == 0 && y == 0); }

	// Returns true if the rectangle contains the given point, inclusive
	inline bool ContainsPoint(int xx, int yy) { return ( (xx >= x) && (xx < (x + w)) && (yy >= y) && (yy < (y + h)) ); }

	void DrawOutline(uint16 rgb15, uint16 *videoRam) {
		// Yes I am lazy.  Why do you ask?
		Rect(x, y, w, 1).DrawFilled(rgb15, videoRam);
		Rect(x, y + 1, 1, h - 1).DrawFilled(rgb15, videoRam);
		Rect(x + w - 1, y + 1, 1, h - 1).DrawFilled(rgb15, videoRam);
		Rect(x, y + h - 1, w, 1).DrawFilled(rgb15, videoRam);
	}

	void DrawFilled(uint16 rgb15, uint16 *videoRam) {
		register int xw = x + w;
		register int yh = y + h;
		register int xx;
		register int yy;
		register uint16 *pt;
		for (xx = x; xx < xw; xx++) {
			for (yy = y; yy < yh; yy++) {
				pt = videoRam + SCREEN_WIDTH * yy + xx;
				*pt = rgb15;
			}
		}
	}
};

#endif // ! RECT_H

