/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef UNDO_H
#define UNDO_H

#include "array.h"


// Size of undo and redo stack
#define MAX_ACTIONS 64


class Canvas;


enum ActionCode {
	AC_NOOP = 0,         // No operation
	AC_COLOR,            // Set pixel color index on a canvas
	ACTIONCODE_LEN
};


struct ActionColor {
	Canvas *mCanvas;
	uint8 mOldIndex;
	uint8 mNewIndex;
	uint16 mX;
	uint16 mY;
};


struct Action {
	ActionCode mCode;
	union {
		ActionColor mColor;
	};
};


class ActionHandler {
private:
	FixedArray<Action, MAX_ACTIONS> mUndoStack;
	FixedArray<Action, MAX_ACTIONS> mRedoStack;

	void DoAction(const Action &action, bool reverse = false);

public:

	// Perform a given action
	void Perform(const Action &action);
	
	bool Undo(void);
	bool Redo(void);
};

#endif // ! UNDO_H
