/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef WIDGET_H
#define WIDGET_H

#include <nds/jtypes.h>
#include "rect.h"

enum PenState {
	PS_JUSTDOWN = 1,  // Pen just came down this frame
	PS_STILLDOWN = 2, // Pen was down the previous frame.  Mutually exclusive with PS_JUSTDOWN
	PS_DOWNAGAIN = 4, // Pen was double-tapped
	PS_DOWN = 8,      // Pen is down (always set with above bits)
	PS_UP =16,        // Pen just went up this frame (mutually exclusive with all above bits)
};


#define WIDGET_DECL(classname) \
private: \
static const uint32 mClassID; \
public: \
virtual uint32 GetClassID(void) const; \
static classname *FromWidget(Widget *widget);

#define WIDGET_IMPL(classname, classid) \
const uint32 classname::mClassID = classid; \
uint32 classname::GetClassID(void) const { return classid; } \
classname * classname::FromWidget(Widget *widget) { return (widget->GetClassID() == classid ? reinterpret_cast< classname *> (widget) : NULL ); }

class Widget {
public:
	// The area this widget occupies in screen pixels
	Rect mArea;
	// Whether or not Render() is called for this widget during the VBLANK interrupt
	bool mVisible;
	// Pointer to parent widget.  This is NULL for the top-level container.
	Widget *mParent;

	Widget(void) : mVisible(false), mParent(NULL) { }
	virtual ~Widget(void) { }

	// Initialize the widget
	virtual void Init(void) { }

	// Cleanup the widget
	virtual void Done(void) { }

	// Frame tick (called every frame)
	virtual void Tick(void) { }

	// Render tick (called _during_ vblank)
	virtual void Render(void) { }

	// Called when a touch event occurs inside a widget's bounds
	virtual void HandleTouch(uint32 state, int16 x, int16 y) { }

	// Return the class ID of this C++ class
	virtual uint32 GetClassID(void) const { return CID_WIDGET; }

};

#endif // ! WIDGET_H

