#!/usr/bin/python

import sys
from array import *

# Make the SRAM header
def MakeHeader():
    header = array('H') # unsigned short array
    header.append(0x4453); # "DS"
    header.append(0x4544); # "ED
    header.append(0x0100) # \
    header.append(0x0000) # /  SRAM version number (0000 0001 in little endian hex)

    for i in range(4, 32): # pad the rest of 64 bytes for the header
        header.append(0)

    return header

# Utility function from libnds
def RGB15(r, g, b):
    return r | g << 5 | b << 10

# Make the default palette
def MakePalette():
    palette = array('H') # unsigned short array
    rgb = [0, 0, 0]
    rgbindex = 0

    # A few sensible defaults
    palette.append(RGB15(31, 0, 31)) # Hot pink for invisible index
    palette.append(RGB15(0, 0, 0)) # Black
    palette.append(RGB15(31, 31, 31)) # White
    palette.append(RGB15(31, 0, 0)) # Red
    palette.append(RGB15(0, 31, 0)) # Green
    palette.append(RGB15(0, 0, 31)) # Blue
    palette.append(RGB15(31, 31, 0)) # Yellow
    palette.append(RGB15(0, 31, 31)) # Cyan

    # Dummy generation sequence for now
    for i in range(8, 256):
        palette.append(RGB15(rgb[0], rgb[1], rgb[2]))
        rgb[rgbindex] = rgb[rgbindex] + 2
        if rgb[rgbindex] > 31:
            rgbindex = rgbindex + 1
            if rgbindex > 2:
                rgbindex = 0
                rgb[rgbindex] = 0

    # This is a good start for generating a greyscale
    #for i in range(8, 256):
    #    palette.append(RGB15(rgb[0], rgb[1], rgb[2]))
    #    rgb[rgbindex] = (rgb[rgbindex] + 1) % 31
    #    rgbindex = rgbindex + 1
    #    if rgbindex > 2:
    #        rgbindex = 0


    return palette

# Make a dummy array of 64 bytes of 0
def MakePadding():
    pad = array('H') # unsigned short array
    for i in range(0, 32):
        pad.append(0)

    return pad



if __name__ == '__main__':
    if len(sys.argv) != 2:
        print "Usage: ", sys.argv[0], "<filename>"
        sys.exit(1)

    try:
        f = file(sys.argv[1], 'w')
    except:
        print "Couldn't open file", sys.argv[1]
        sys.exit(1)

    header = MakeHeader()
    header.byteswap()
    sram = header + MakePalette();
    pad = MakePadding()
    # 1015 tiles + 64K padding
    for i in range(0, 2039):
        sram = sram + pad

    sram.write(f)
    f.close()