/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <nds.h>
#include "dusted.h"
#include "tilewin.h"
#include "palette.h"
#include "engine.h"
#include "canvas.h"

WIDGET_IMPL(TileEditorWindow, CID_TILEEDITORWINDOW);

#define SCREEN_TILE_INDEX(x, y) ((y) * 32 + (x))

void TileEditorWindow::Init(void) {
	Window::Init();

	// Create the palette widget and initialzie it
	mPaletteWidget = new Palette;
	mPaletteWidget->mParent = this;
	mPaletteWidget->mArea.Set(192, 0, 64, 192);
	mPaletteWidget->Init();
	mWidgets.Push(mPaletteWidget);

	// Create the canvas widget and initialize it
	for (int i = 0; i < NUM_CANVAS; i++) {
		mCanvas[i] = new Canvas;
		mCanvas[i]->mParent = this;
		mCanvas[i]->Init();
		if (i > 0) {
			mCanvas[i]->SetArea((i % 3) * 64, (i / 3) * 64, 64, 64);
		}
		mCanvas[i]->mVisible = false;
		mWidgets.Push(mCanvas[i]);
	}

	// Default to single tile mode
	mMode = 0;
	mCanvas[0]->SetArea(0, 0, 192, 192);
	mCanvas[0]->mVisible = true;
	// Default to the tile view
	mSubView = 1;
	//HideConsole();
	//mUpdateSubView = true;
}


void TileEditorWindow::Tick(void) {
	// Let the pen input pass between canvases
	if (mPenWasDown && keysHeld() & KEY_TOUCH) {
		int canvasHasFocus = -1, canvasNeedsFocus = -1;
		for (int i = 0; i < NUM_CANVAS; i++) {
			if (!mCanvas[i]->mVisible) {
				continue;
			}

			if (mCanvas[i] == mFocusWidget) {
				canvasHasFocus = i;
			}
			if (mCanvas[i]->mArea.ContainsPoint(IPC->touchXpx, IPC->touchYpx)) {
				canvasNeedsFocus = i;
			}
			if (canvasHasFocus != -1 && canvasHasFocus == canvasNeedsFocus) {
				canvasHasFocus = -1;
				break;
			}
		}
		//simWarning("has %i, needs %i\n", canvasHasFocus, canvasNeedsFocus);
		if (canvasHasFocus != -1 && canvasNeedsFocus != -1) {
			mFocusWidget = mCanvas[canvasNeedsFocus];
		}
	}
	
	// Run window tick first so it processes the keys variables
	Window::Tick();
	
	//TODO this is hideously inefficient, fix it
	if (mMode == 1) {
		for (int i = 0; i < NUM_CANVAS; i++) {
			mCanvas[i]->Refresh();
		}
	}

	//TODO make this repeat every second instead of being strictly on-down only
	if (keysDown() & KEY_LEFT) {
		mActions.Undo();
		UpdateSubView();
	} else if (keysDown() & KEY_RIGHT) {
		mActions.Redo();
		UpdateSubView();
	} else if (keysDown() & KEY_UP) {
		gEngine->ReadPBuffer(576 + mCanvas[0]->GetTile() * 64, 64); // Read the current tile
		mCanvas[0]->Refresh();
		UpdateSubView();
	} else if (keysDown() & KEY_DOWN) {
		gEngine->WritePBuffer(576 + mCanvas[0]->GetTile() * 64, 64); // Write the current tile
	} else if (keysDown() & KEY_A) {
		// Toggle the mode
		//TODO implement four-tile mode
		mMode++;
		if (mMode > 3) {
			mMode = 0;
		}
		if (mMode == 0) {
			mCanvas[0]->SetArea(0, 0, 192, 192);
		} else if (mMode == 1) {
			int baseTile = mCanvas[0]->GetTile();
			for (int i = 0; i < 9; i++) {
				mCanvas[i]->SetTile(baseTile);
				mCanvas[i]->SetArea(64 * (i % 3), 64 * (i / 3), 64, 64);
			}
			for (int i = 1; i < 9; i++) {
				mCanvas[i]->SetTile(baseTile);
			}
		} else if (mMode == 2) {
			int baseTile = mCanvas[0]->GetTile();
			for (int i = 0; i < 4; i++) {
				mCanvas[i]->SetTile(baseTile + i);
				mCanvas[i]->SetArea(96 * (i % 2), 96 * (i / 2), 96, 96);
			}
		} else if (mMode == 3) {
			mCanvas[0]->SetArea(0, 0, 64, 64);
			int baseTile = mCanvas[0]->GetTile();
			for (int i = 0; i < 9; i++) {
				mCanvas[i]->SetTile(baseTile + i);
				mCanvas[i]->SetArea(64 * (i % 3), 64 * (i / 3), 64, 64);
			}
		}

		for (int i = 1; i < NUM_CANVAS; i++) {
			if (mMode == 0) {
				mCanvas[i]->mVisible = false;
			} else if (mMode == 2) {
				mCanvas[i]->mVisible = (i < 4);
				if (mCanvas[i]->mVisible) {
					mCanvas[i]->Refresh();
				}
			} else if (mMode == 1 || mMode == 3) {
				mCanvas[i]->mVisible = true;
				mCanvas[i]->Refresh();
			}
		}
		UpdateSubView();
	} else if (keysDown() & KEY_L) {
		for (int i = 0; i < NUM_CANVAS; i++) {
			mCanvas[i]->PrevTile();
		}
		UpdateSubView();
	} else if (keysDown() & KEY_R) {
		for (int i = 0; i < NUM_CANVAS; i++) {
			mCanvas[i]->NextTile();
		}
		UpdateSubView();
	} else if (keysDown() & KEY_START) {
		if (mSubView == 0) {
			HideConsole();
		}
		mSubView++;
		if (mSubView > 2) {
			mSubView = 0;
			mUpdateSubView = false;
			ShowConsole();
		} else {
			mUpdateSubView = true;
		}
	}
}


void TileEditorWindow::Render(void) {
	Window::Render();
	if (mUpdateSubView) {
		u8 *charBase = (u8 *) CHAR_BASE_BLOCK_SUB(0);
		u16 *screenBase = (u16 *) SCREEN_BASE_BLOCK_SUB(31);
		u16 maxTiles = 0;

		// Clear out the first ten tiles
		memset(charBase, 0, 5120); //TODO should be 640?
	
		SUB_BG0_CR = BG_256_COLOR | BG_MAP_BASE(31);

		// Copy the palette into the subview's palette
		memcpy(BG_PALETTE_SUB, mPaletteWidget->mPalette, 512);
		
		// Clear index map space
		memset(screenBase, 0, 32 * 1024);

		switch (mSubView) {
			case 1: // Non-repeating
				charBase += 64;
				
				switch (mMode) {
					case 0: // Single tile
						screenBase[400] = 1;
						maxTiles = 1;
						break;
					case 1: // Tile borders
						screenBase[367] = 1;
						screenBase[368] = 1;
						screenBase[369] = 1;
						screenBase[399] = 1;
						screenBase[400] = 1;
						screenBase[401] = 1;
						screenBase[431] = 1;
						screenBase[432] = 1;
						screenBase[433] = 1;
						maxTiles = 1;
					case 2: // Four tile
						screenBase[367] = 1;
						screenBase[368] = 2;
						screenBase[399] = 3;
						screenBase[400] = 4;
						maxTiles = 4;
						break;
					case 3: // Nine tile
						screenBase[367] = 1;
						screenBase[368] = 2;
						screenBase[369] = 3;
						screenBase[399] = 4;
						screenBase[400] = 5;
						screenBase[401] = 6;
						screenBase[431] = 7;
						screenBase[432] = 8;
						screenBase[433] = 9;
						maxTiles = 9;
						break;
				}
				
				// Copy the current tiles into the tile memory
				for (int i = 0; i < maxTiles; i++) {
					memcpy(charBase, mCanvas[i]->mPixels, 64);
					charBase += 64;
				}

				break;
			case 2: // Repeating
				
				switch (mMode) {
					case 0: // Single tile
					case 1: // Tile borders
						// These are the same in repeating mode
						maxTiles = 1;
						break;
					case 2: // Four tile
						charBase += 64;

						for (int x = 0; x < 32; x++) {
							for (int y = 0; y < 24; y++) {
								screenBase[SCREEN_TILE_INDEX(x, y)] =	(y & 1 ? 3 : 1) + (x % 2);
							}
						}
						
						maxTiles = 4;
						break;
					case 3: // Nine tile
						charBase += 64;

						for (int x = 0; x < 30; x++) { // Last two columns must be left empty
							for (int y = 0; y < 24; y++) {
								register int offset = y % 3;
								if (offset == 0) {
									offset = 1;
								} else if (offset == 1) {
									offset = 4;
								} else {
									offset = 7;
								}
								screenBase[SCREEN_TILE_INDEX(x, y)] = offset + (x % 3);
							}
						}
						
						maxTiles = 9;
						break;
				}
				
				// Copy the current tiles into the tile memory
				for (int i = 0; i < maxTiles; i++) {
					memcpy(charBase, mCanvas[i]->mPixels, 64);
					charBase += 64;
				}
		}

		mUpdateSubView = false;
	}
}

