/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <nds.h>
#include "dusted.h"
#include "window.h"

WIDGET_IMPL(Window, CID_WINDOW);

// Number of heartbeats it takes to double-tap the touchpad
//#define PEN_EPSILON 10

void Window::Init(void) {
	mPenWasDown = false;
	mFocusWidget = NULL;
	mLastPenDown = 0;
	memset(&mPenXY, 0, sizeof(touchPosition));

	Container::Init();
}


void Window::Tick(void) {
	// Update the key data
	scanKeys();

	// Handle touchpad events
	if (keysHeld() & KEY_TOUCH) {
		touchPosition newXY = touchReadXY();

		uint16 dx = abs(newXY.px - mPenXY.px);
		uint16 dy = abs(newXY.py - mPenXY.py);

		mPenXY = newXY;

		// filtering measurement errors
		if (dx < 8 && dy < 8) {
			LogInfo("dx %u, dy %u\n", dx, dy);

			if (mPenDownTicks < PENDOWN_THRESHOLD) {
				mPenDownTicks++;
			} else {
				uint32 pen;

				pen = PS_DOWN;

				// Determine what state to broadcast
				if (mPenWasDown) {
					pen |= PS_STILLDOWN;
				} else {
					pen |= PS_JUSTDOWN;

					//TODO implement double tap detection
					// See if this is considered a double-tap
					//if (IPC->heartbeat - mLastPenDown < PEN_EPSILON) {
					//	pen |= PS_DOWNAGAIN;
					//}

					// Record the last down event time
					mLastPenDown = IPC->unixTime;

					// Find the widget (if any) to handle the pen event.  This widget will
					// continue to receive pen events until the pen up, even if the pen leaves the widget's area.
					for (uint32 i = 0; i < mWidgets.Size(); i++) {
						if (mWidgets[i]->mVisible && mWidgets[i]->mArea.ContainsPoint(IPC->touchXpx, IPC->touchYpx)) {
							mFocusWidget = mWidgets[i];
							break;
						}
					}

					// Note that the pen is down
					mPenWasDown = true;
				}

				// Send the touch event to the focus widget, if any.
				if (mFocusWidget) {
					mFocusWidget->HandleTouch(pen, IPC->touchXpx, IPC->touchYpx);
				}
			}
		} else {
			LogInfo("OUT OF BOUNDS:  dx %u, dy %u\n", dx, dy);
		}
	} else {
		if (mPenWasDown) { // Handle pen up event
			if (mFocusWidget) {
				mFocusWidget->HandleTouch(PS_UP, -1, -1);
			}

			mPenWasDown = false;
			mFocusWidget = NULL;
		}

		// In either case, clear the first pen down marker
		mPenDownTicks = 0;
	}

	// Frame tick each widget
	Container::Tick();
}

